package cl.axg.hls_player;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.res.Configuration;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import com.google.android.exoplayer2.ExoPlaybackException;
import com.google.android.exoplayer2.Player;
import com.google.android.exoplayer2.SimpleExoPlayer;
import com.google.android.exoplayer2.source.MediaSource;
import com.google.android.exoplayer2.source.ProgressiveMediaSource;
import com.google.android.exoplayer2.source.dash.DashMediaSource;
import com.google.android.exoplayer2.source.hls.HlsMediaSource;
import com.google.android.exoplayer2.trackselection.DefaultTrackSelector;
import com.google.android.exoplayer2.ui.PlayerView;
import com.google.android.exoplayer2.upstream.DefaultHttpDataSource;
import com.google.android.exoplayer2.upstream.DefaultHttpDataSourceFactory;
import com.google.android.exoplayer2.util.Util;

import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.Objects;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

public class MainActivity extends Activity {

    private static final String TAG = "MainActivity";
    private String URL_VIDEO= null;
    PlayerView playerView;
    SimpleExoPlayer player;
    Handler mHandler = new Handler();
    Runnable mRunnable;
    boolean hayError;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        Bundle extras = getIntent().getExtras();
        if (extras != null){
            URL_VIDEO  =(String) extras.get("url");
        }else{
            Log.d(TAG, "Sin URL video exoplayer");
            throw new IllegalArgumentException("Sin URL video exoplayer");

        }

        setContentView(R.layout.activity_main);
        ignoreSSL();
        setUp();
    }
    private void setUp() {
        initializePlayer();
    }


    private void initializePlayer() {
        playerView = this.findViewById(R.id.playerView);
        playerView.requestFocus();
        if(mHandler!=null)
            mHandler.removeMessages(0);
        DefaultTrackSelector trackSelector = new DefaultTrackSelector(this);
        trackSelector.setParameters(trackSelector.buildUponParameters()
                .setMaxAudioChannelCount(1)
        );
        player = new SimpleExoPlayer.Builder(this).setTrackSelector(trackSelector).build();
        playerView.setPlayer(player);
        playerView.setShowBuffering(PlayerView.SHOW_BUFFERING_ALWAYS);
        player.addListener(new Player.EventListener() {
            @Override
            public void onPlayerError(ExoPlaybackException error) {

                switch (error.type) {
                    case ExoPlaybackException.TYPE_SOURCE:
                        Log.e(TAG, "TYPE_SOURCE: " + error.getSourceException().getMessage());
                        stop();
                        break;

                    case ExoPlaybackException.TYPE_RENDERER:
                        Log.e(TAG, "TYPE_RENDERER: " + error.getRendererException().getMessage());
                        stop();
                        break;

                    case ExoPlaybackException.TYPE_UNEXPECTED:
                        Log.e(TAG, "TYPE_UNEXPECTED: " + error.getUnexpectedException().getMessage());
                        stop();
                        break;

                    default:
                        mHandler.postDelayed(new Runnable() {
                            @Override
                            public void run() {
                                hayError = true;
                                pause();
                                reproducir();
                            }
                        }, 1000 * 10);
                        break;
                }

            }
        });

        player.setPlayWhenReady(true);
        play(player);
    }

    public void pause(){
        player.setPlayWhenReady(false);
        hayError = false;
    }


    public void stop(){
        player.setPlayWhenReady(false);
        player.stop();
        player.seekTo(0);
    }

    public void play(SimpleExoPlayer player){
        try{
            prepareMediaForPlaying(Uri.parse(URL_VIDEO));
            player.setPlayWhenReady(true);
            hayError = false;
        }catch (Exception ex){
            Log.e(TAG, "ERROR REPRODUCTOR MAIN ACTIVITY: "+ex.getMessage());
            Toast.makeText(this, "ERROR REPRODUCTOR URL",  Toast.LENGTH_LONG).show();
        }

    }

    public void reproducir(){

        prepareMediaForPlaying(Uri.parse(URL_VIDEO));
        player.setPlayWhenReady(true);
        hayError = false;
    }
    private void prepareMediaForPlaying(Uri mediaSourceUri) {

        DefaultHttpDataSourceFactory dataSourceFactory = new DefaultHttpDataSourceFactory(
                Util.getUserAgent(this, "VideoPlayerGlue"),
                DefaultHttpDataSource.DEFAULT_CONNECT_TIMEOUT_MILLIS,
                DefaultHttpDataSource.DEFAULT_READ_TIMEOUT_MILLIS,
                true);
        MediaSource    mediaSource = getMediaSource(mediaSourceUri , dataSourceFactory);
        player.prepare(mediaSource);
        player.setPlayWhenReady(true);

    }
    private MediaSource getMediaSource(Uri uri,DefaultHttpDataSourceFactory dataSourceFactory){
        String[] extension = Objects.requireNonNull(uri.getPath()).split("\\.");
        int largo = extension.length;
        if(largo>1){
            switch (extension[largo-1].toUpperCase()){
                case "MPD":
                    return new DashMediaSource
                            .Factory(dataSourceFactory)
                            .createMediaSource(uri);
                case "M3U8":
                    return new HlsMediaSource
                            .Factory(dataSourceFactory)
                            .createMediaSource(uri);


            }
        }
        return new ProgressiveMediaSource.Factory(dataSourceFactory).
                createMediaSource(uri);
    }



    public void closeImgBtnOnClick(View v)
    {
        this.finish();
    }

    private void releasePlayer() {
        if (player != null) {
            player.release();
            player = null;
        }
    }
    private void pausePlayer() {
        if (player != null) {
            player.setPlayWhenReady(false);
            player.getPlaybackState();
        }
    }
    private void resumePlayer() {
        if (player != null) {
            player.setPlayWhenReady(true);
            player.getPlaybackState();
        }
    }
    @Override
    protected void onPause() {
        super.onPause();
        pausePlayer();
        if (mRunnable != null) {
            mHandler.removeCallbacks(mRunnable);
        }
    }
    @Override
    protected void onRestart() {
        super.onRestart();
        resumePlayer();
    }
    @Override
    protected void onDestroy() {
        super.onDestroy();
        releasePlayer();
    }


    private void ignoreSSL(){
        try {
            TrustManager[] victimizedManager = new TrustManager[]{

                    new X509TrustManager() {

                        public X509Certificate[] getAcceptedIssuers() {

                            return new X509Certificate[0];
                        }

                        @SuppressLint("TrustAllX509TrustManager")
                        @Override
                        public void checkClientTrusted(X509Certificate[] certs, String authType) {
                        }

                        @SuppressLint("TrustAllX509TrustManager")
                        @Override
                        public void checkServerTrusted(X509Certificate[] certs, String authType) {
                        }
                    }
            };

            SSLContext sc = SSLContext.getInstance("SSL");
            sc.init(null, victimizedManager, new SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
            HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
                @SuppressLint("BadHostnameVerifier")
                @Override
                public boolean verify(String s, SSLSession sslSession) {
                    return true;
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        // ignore orientation/keyboard change
        super.onConfigurationChanged(newConfig);
    }

    @Override
    public void onBackPressed() {
        Log.d("CDA", "onBackPressed Called");
        super.onBackPressed();
    }

}